(* use_altp.ml: Demonstrate how types defined in modules are available
   elsewhere and can co-exist with existing types like the list and
   option types defined in Pervasives. *)

let a  = Altp.Apple;;                   (* Demo unique type fruit  *)
let g3 = Altp.Grapes 3;;                (* which must be qualified  *)
(* val a  : Altp.fruit *)
(* val g3 : Altp.fruit *)

let aplst =                             (* alternate pervasives list, gross *)
  Altp.(::) (5, (Altp.(::) (6, Altp.([]))))
;;
(* val aplist : int Altp.list *)

let splist : int list =                 (* standard list *)
  5 :: 6 :: []
;;
(* val splist : int list *)

let apopt = Altp.Some true;;            (* alternate pervasives option *)
(* val apopt : boolean Altp.option *)

let spopt = Some true;;                 (* standard option *)
(* val spopt : boolean option *)

let rm =                                (* named pair *)
  {Altp.first="Rick"; Altp.second="Morty"}
;;
(* val rm : string * string Altp.namedpair *)

let r = rm.Altp.first;;                 (* qualified field name: awkward *)
(* "Rick" *)

let m = rm.second;;                     (* unqualified field name *)
(* "Morty" *)
(* unqualified field name may generate warning/error in older OCaml *)

open Altp;;                             (* open Altp: resolve bindings from Altp *)

let o = Orange;;                        (* don't need to qualify after open *)
(* val o : Altp.fruit *)

let aplst2 = 5::6::[];;                 (* :: and [] are now from Altp *)
(* val aplst2 : int Altp.list *)

(* Earliear list and this one are equivalent *)
Printf.printf "(aplst = aplst2) : %b\n" (aplst = aplst2);;

let apopt2 = Some true;                 (* Some is no from Altopt *)
(* val apopt2 : boolean Altp.option *)



