(* use_mapmake.ml: demonstrate usage of Map.Make to create maps from
   various key types to values set. *)
open Printf;;

(* interface for string key to Map.Make *)
module StringKey = struct
  type t = string;; (* elem type *)
  let compare =     (* comparison *)
    String.compare
  ;;
end;;

(* call the functor, get a module *)
module StringMap = Map.Make(StringKey);;

(* use the new module *)
let _ =
  (* string key, int value  *)
  let map1 = StringMap.empty in
  let map2 = StringMap.add "Morty" 80  map1 in
  let map3 = StringMap.add "Rick"  190 map2 in
  let map4 = StringMap.add "Jerry" 35  map3 in
  StringMap.iter (printf "%s->%d\n") map4;

  (* string key, string value *)
  let map2 =
    StringMap.add "Morty" "Smith"   map1 in
  let map3 =
    StringMap.add "Rick"  "Sanchez" map2 in
  let map4 =
    StringMap.add "Jerry" "Smith"   map3 in
  StringMap.iter (printf "%s->%s\n") map4;
;;

(* interface for int key to Map.Make *)
module IntKey = struct
  type t = int;;    (* elem type *)
  let compare x y = (* comparison *)
    x - y
  ;;
end;;

(* call the functor, get a module *)
module IntMap = Map.Make(IntKey);;

(* use the new module *)
let _ =
  (* int key, string value *)
  let map1 = IntMap.empty in
  let map2 = IntMap.add 80  "Morty" map1 in
  let map3 = IntMap.add 190 "Rick"  map2 in
  let map4 = IntMap.add 35  "Jerry" map3 in

  IntMap.iter (printf "%d->%s\n") map4;
;;
