(* use_setmake.ml: demonstrate usage of Set.Make to create an integer
   set. Demonstrate the set's persistent characteristics. *)
open Printf;;

(* create module for input to 
   functor  *)
module IntElem = struct
  type t = int;;    (* elem type *)
  let compare x y = (* comparison *)
    x - y
  ;;
end;;

(* call the functor, get a module *)
module IntSet = Set.Make(IntElem);;

(* use the new module *)
let _ =
  let set1 = IntSet.empty in
  let set2 = IntSet.add 50 set1 in
  let set3 = IntSet.add 75 set2 in
  let set4 = IntSet.add 65 set3 in

  IntSet.iter (fun i->printf "%d " i) set4;
  printf "\n";

  let int_list = [22; 56; 99; 11;
                  33; 44; 34; 89]
  in
  let set5 =
    let help set i =
      IntSet.add i set
    in
    List.fold_left help set4 int_list
  in
  IntSet.iter (fun i->printf "%d " i) set5;
  printf "\n";
  let sum = IntSet.fold (+) set5 0 in
  printf "sum: %d\n" sum;
;;

(* interface strings with Set.Make *)
module StringElem = struct
  type t = string;;
  let compare = String.compare;;
end;;

(* call the functor, get a module *)
module StringSet =
  Set.Make(StringElem);;

let strset1 =
  StringSet.add "hello"
                StringSet.empty;;

(* a brand new record type *)
type strint = {
    str : string;
    num : int;
  };;

(* interface string with Set.Make *)
module StrintElem = struct
  type t = strint;;
  let compare x y =
    let diff =
      String.compare x.str y.str in
    if diff<>0 then
      diff
    else
      x.num - y.num
  ;;
end;;

(* call the functor, get a module *)
module StrintSet =
  Set.Make(StrintElem);;

let strintset1 =
  StrintSet.add {str="hi";num=5}
                StrintSet.empty;;
